#define F_CPU 16000000
#include <avr/io.h>
#include <util/delay.h>
#include <avr/interrupt.h>

#define MIN_SAFE_DISTANCE 10
#define MAX_BEEP_DISTANCE 100
#define HC_SR04_NUM_DEVICES	2

static volatile uint8_t flag = 0;

static volatile uint8_t count = 0;
static volatile uint8_t count1 = 0;

static volatile uint8_t saved_tcnt0 = 0;
static volatile uint8_t saved_count = 0;

static volatile uint8_t saved_tcnt0_1 = 0;
static volatile uint8_t saved_count_1 = 0;

static volatile uint8_t left = 1;
static volatile uint8_t right = 1;

static volatile float saved_distance = 0.0f;

static volatile int selected_sensor;

void play_intermitent_tone(int total_duration) {
	for (int i = 0; i < 50; i++) {
		
		if (left != 0) {
			right = 0;
			PORTA |= (1 << PA7);
			PORTA |= (1 << PA5);
			_delay_ms(1);
			PORTA &= ~(1 << PA7);
			PORTA &= ~(1 << PA5);
			_delay_ms(1);
		}
		if (right != 0) {
			left = 0;
			PORTA |= (1 << PA6);
			PORTA |= (1 << PA4);
			_delay_ms(1);
			PORTA &= ~(1 << PA6);
			PORTA &= ~(1 << PA4);
			_delay_ms(1);
		}
	}
	for (int i = 0; i < total_duration; i++) {
		_delay_ms(1);
	}
}

ISR(TIMER0_OVF_vect, ISR_NOBLOCK) // Other interrupts aren't blocked during this routine
{
	count++;
	count1++;
}

ISR(PCINT0_vect, ISR_NOBLOCK)
{
	if (selected_sensor == 0) {
		if ((PINA & (1 << PA1)) != 0) {
			/* Echo rising edge */
			TCNT0 = 0;
			count = 0;
			left = 0;
		}
		else {
			/* Echo falling edge */
			saved_tcnt0 = TCNT0;
			saved_count = count;
			flag = 1;
			left = 1;
			right = 0;
		}
	}
	if (selected_sensor == 1) {
		if ((PINA & (1 << PA3)) != 0) {
			/* Echo rising edge */
			TCNT0 = 0;
			count1 = 0;
			right = 0;
		}
		else {
			/* Echo falling edge */
			saved_tcnt0_1 = TCNT0;
			saved_count_1 = count1;
			flag = 1;
			right = 1;
			left = 0;
		}
	}
}

void HC_SR04_init()
{
	TCCR0A = 0;
	TCCR0B = 0;
	TIMSK0 = 0;

	sei(); // Enable global interrupts
	TCCR0B |= (1 << CS02); // Prescaler = 256 => T = 16us
	TIMSK0 |= (1 << TOIE0); // Enable Overflow interrupt

	/* Configure pins for trigger and echo */
	DDRA |= (1 << PA0); // Trigger
	DDRA &= ~(1 << PA1); // Echo

	PORTA &= ~(1 << PA0);

	DDRA |= (1 << PA2); // Trigger
	DDRA &= ~(1 << PA3); // Echo

	PORTA &= ~(1 << PA2);

	/* Enable pin change interrupt on echo pin */
	PCICR |= (1 << PCIE0);
	PCMSK0 = 0;
	PCMSK0 |= (1 << PCINT1);

	//PCICR |= (1 << PCIE0);
	// PCMSK0 = 0;
	// PCICR |= (1 << PCIE2);
	PCMSK0 |= (1 << PCINT3);
}

float HC_SR04_get_distance(int device)
{
	double sum = 0;
	flag = 0;

	selected_sensor = device;

	if (selected_sensor == 0) {
		/* Reset Trigger signal for a clean rise */
		PORTA &= ~(1 << PA0);
		_delay_us(5);

		/* Send Trigger signal for > 10us */
		PORTA |= (1 << PA0);
		_delay_us(50);
		PORTA &= ~(1 << PA0);

		// _delay_ms(5);
		while (flag == 0 && count < 5);

		sum = sum + (((int)saved_tcnt0 + 255 * saved_count) * 16.0) * 0.017;
	}
	if (selected_sensor == 1) {
		/* Reset Trigger signal for a clean rise */
		PORTA &= ~(1 << PA2);
		_delay_us(5);

		/* Send Trigger signal for > 10us */
		PORTA |= (1 << PA2);
		_delay_us(50);
		PORTA &= ~(1 << PA2);

		// _delay_ms(5);
		while (flag == 0 && count1 < 5);

		sum = sum + (((int)saved_tcnt0_1 + 255 * saved_count_1) * 16.0) * 0.017;
	}

	/* Wait for the Echo */

	//selected_sensor = -1;
	return sum;
}


int main() 
{
	HC_SR04_init();
	DDRA |= (1 << PA7);
	PORTA &= ~(1 << PA7);
	DDRA |= (1 << PA6);
	PORTA &= ~(1 << PA6);
	
	DDRA |= (1 << PA4);
	PORTA &= ~(1 << PA4);
	DDRA |= (1 << PA5);
	PORTA &= ~(1 << PA5);


	while (1) {
		int dist = (int)HC_SR04_get_distance(0);
		_delay_ms(10);
		int dist2 = (int)HC_SR04_get_distance(1);

		int min = dist < dist2 ? dist : dist2;
		if (min < MAX_BEEP_DISTANCE) {
			if (min < MIN_SAFE_DISTANCE) {
				play_intermitent_tone(0);
			}
			else {
				play_intermitent_tone(min * 2);
			}
		}
	}

	return 0;
}
